"""
Simple interface to the urlscan.io API


Author: Mark Kendrick (github.com/aeetos | linkedin.com/in/mkendrick)


MIT License

Copyright (c) Microsoft Corporation. All rights reserved.

MIT License Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE



Usage:

# Import this libary
import urlscanio

# Create an API instance with your API key
urlscanio_api = urlscanio.URLScanAPI('abc-def')

# Query the search endpoint - see urlscan.io API docs for supported operators and fields
urlscanio_results = urlscanio_api.search('domain:riskiq.net')

# Results object is list-like
len(urlscanio_results)

# Pandas is supported; use as_df to get a dataframe of results
urlscanio_results.as_df

# Each result is an object with fields; try the as_dict property to get a flat dictionary representation
urlscanio_results[0].as_dict

# Each result is an object with fields, and includes support for Jupyter display() functions
# Try this in a notebook cell:
for urlscanio_result in urlscanio_results:
    display(urlscanio_result)


"""



from datetime import datetime
import requests

BASE_URL = 'https://urlscan.io/api/v1'

def parse_urlscanio_date(date_string):
    # %f in the format string may not be correct here
    return datetime.strptime(date_string, '%Y-%m-%dT%H:%M:%S.%fZ')


class URLScanAPI:
    
    def __init__(self, api_key, base_url=BASE_URL):
        self._api_key = api_key
        self._base_url = base_url
        self._last_result = {}
        self._session = requests.Session()
        self._session.headers.update({
            'Content-Type':'application/json',
            'API-Key': api_key
        })
    
    @property
    def last_search_results(self):
        return self._last_search_results
    
    def get(self, endpoint, **kwargs):
        return self._session.get('/'.join((self._base_url, endpoint)), **kwargs)
    
    def search(self, query, params={}):
        params.update({'q':query})
        result = self.get('search', params=params)
        result.raise_for_status()
        self._last_search_results_raw = result.json()
        self._last_search_results = UrlScanSearchResults(self._last_search_results_raw, query)
        return self._last_search_results



class UrlScanSearchResults:

    def __init__(self, results, query=''):
        self._raw_results = results
        self._query = query
        self._results = [ UrlScanSearchResult(r, query) for r in results.get('results', []) ]

    def __len__(self):
        return len(self._results)
    
    def __getitem__(self, key):
        return self._results.__getitem__(key)
    
    def __iter__(self):
        return self._results.__iter__()

    @property
    def raw_results(self):
        return self._raw_results
    
    @property
    def as_df(self):
        from pandas import concat
        return concat([ r.as_df for r in self])



class UrlScanSearchResult:

    def __init__(self, result, query):
        self._raw_result = result
        self._query = query
    
    @property
    def as_dict(self):
        d = {
            'indexed_at': self.indexed_at,
            'result_url': self.result_url,
            'screenshot_url': self.screenshot_url,
            'search_query': self._query,
        }
        d.update({
            'task_{}'.format(k) : v for k,v in self.task.items()
        })
        d.update({
            'stats_{}'.format(k) : v for k,v in self.stats.items()
        })
        d.update({
            'page_{}'.format(k) : v for k,v in self.page.items()
        })
        return d
    
    @property
    def as_df(self):
        from pandas import DataFrame
        return DataFrame.from_records([self.as_dict])

    @property
    def raw_result(self):
        return self._raw_result

    @property
    def indexed_at(self):
        return parse_urlscanio_date(self._raw_result['indexedAt'])
    
    @property
    def task(self):
        task = self._raw_result.get('task', None)
        if task is None:
            return {}
        if isinstance(task['time'], str):
            task.update({'time_raw': task['time']})
            task.update({'time': parse_urlscanio_date(task['time_raw'])})
        return task
    
    @property
    def stats(self):
        return self._raw_result.get('stats', {})
    
    @property
    def page(self):
        return self._raw_result.get('page', {})
    
    @property
    def result_url(self):
        return self._raw_result.get('result', '')
    
    @property
    def screenshot_url(self):
        return self._raw_result.get('screenshot', '')
    
    @property
    def screenshot_html(self):
        return '<img src="{}">'.format(self.screenshot_url)
    
    @property
    def screenshot_image(self):
        from IPython.display import Image
        return Image(url=self.screenshot_url)
    
    def _repr_html_(self):
        fields = { k:self.as_dict.get(k,'-') for k in [
                'task_domain','task_url','task_time_raw','task_visibility','task_uuid',
                'page_url','page_country','page_city','page_ip','page_asn','page_asnname',
                'page_server','page_mimeType','page_status',
                'stats_uniqIPs','stats_consoleMsgs','stats_uniqCountries','stats_requests',
                'result_url','screenshot_url','search_query'
            ]
        }
        return """
        <style>
            DIV.rpt-label {{
                font-weight: bold;
                font-size: 10pt;
                margin-top: 5px;
            }}
            .stat {{
                font-family: monospace;
                font-weight: bold;
            }}
            .mono {{
                font-family: monospace;
            }}
        </style>
        <div style="display:block">
            <div style="float:left;width:50%">
                <h3>URLScan.io Search Result</h3>
                <div class="rpt-label">Query</div>
                <div class="mono">{search_query}</div>
                <div class="rpt-label">Domain</div>
                <div class="mono">{task_domain}</div>
                <div class="rpt-label">Scan URL</div>
                <div><textarea readonly cols=50 rows=2>{task_url}</textarea></div>
                <div class="rpt-label">Page URL</div>
                <div><textarea readonly cols=50 rows=2>{page_url}</textarea></div>
                <div class="rpt-label">Scan Task Date/Time</div>
                <div>{task_time_raw}</div>
                <div>
                    <div style="float:left;width:50%">
                        <div class="rpt-label">Scan Stats</div>
                        <div>
                            <div>Unique IPs: <span class="stat">{stats_uniqIPs}</span></div>
                            <div>Console Msgs: <span class="stat">{stats_consoleMsgs}</span></div>
                            <div>Unique Countries: <span class="stat">{stats_uniqCountries}</span></div>
                            <div>Requests: <span class="stat">{stats_requests}</span></div>
                            <div>Visibility: <span class="stat">{task_visibility}</span></div>
                        </div>
                    </div>
                    <div style="float:left;width:50%;">
                        <div class="rpt-label">Page Stats</div>
                        <div>
                            <div>Geo: <span class="stat">{page_country} - {page_city}</span></div>
                            <div>IP: <span class="stat">{page_ip}</span></div>
                            <div>ASN: <span class="stat">{page_asn} ({page_asnname})</span></div>
                            <div>IP: <span class="stat">{page_server}</span></div>
                            <div>MIME type: <span class="stat">{page_mimeType}</span></div>
                            <div>Status code: <span class="stat">{page_status}</span></div>
                        </div>
                    </div>
                </div>
                
                <div class="rpt-label">Report ID <span style="font-weight:normal">(click to view at urlscan.io)</span></div>
                <div class="mono"><a target="_blank" href="https://urlscan.io/result/{task_uuid}/">{task_uuid}</a></div>
            </div>
            <div style="float:left;width:50%">
                <img width="100%" src="{screenshot_url}" style="border:1px solid #AAA">
            </div>
        </div>
        """.format(**fields)
    
